/***************************************************************************
 *            element.h
 *
 *  Tue Dec 21 15:40:34 2004
 *  Copyright  2004  Jean-Pierre Lozi
 *  jean-pierre@lozi.org - http://gtuxball.lozi.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef ELEMENT_H
#define ELEMENT_H

/* Shape types */
#define BLOCK			0
#define BALL			1
#define PAD				2

/* Attributes */
#define ELEMENT_TYPE	0
#define ELEMENT_X		1
#define ELEMENT_Y		2
#define ELEMENT_WIDTH	3
#define ELEMENT_HEIGHT	4
#define ELEMENT_COLOR	5

/* Colors */
#define RED					1
#define GREEN				2
#define BLUE				3
#define CYAN				4
#define MAGENTA				5
#define YELLOW				6
#define PURPLE				7
#define NUMBER_OF_COLORS	7

/* Pad types */
#define PAD1_LEFT				0
#define PAD2_LEFT				1
#define PAD1_RIGHT				2
#define PAD2_RIGHT				3

/* Those variables contain the links to the different elements' pictures. */
extern const char * blocks_image_files[];
extern const char * pad_image_files[];
extern const char * ball_image_file;

/* Miscellaneous constants */
#define BALL_DIAMETER			18
#define BALL_INITIAL_X			322
#define BALL_INITIAL_Y			330
#define BLOCK_WIDTH				75
#define BLOCK_HEIGHT			25
#define BLOCK_MARGIN			5
#define PAD_INITIAL_X			270
#define PAD_INITIAL_Y			425
#define PAD_WIDTH				105
#define PAD_HEIGHT				54

/* Abstract type */
typedef struct element *Element;
typedef void ( *Element_draw_function )( Element el, void * data );

/* Public functions : */

/* This functions creates and returns an Element of the given type. */
extern Element create_element( int type );
/* This function destroys the given element. */
extern void element_destroy ( Element *obj );
/* This function allows to modify the element's integer attributes. */
extern void element_set_attribute( Element obj, int attribute, int value );
/* This function allows to modify the element's data. */
void element_set_data( Element obj, void * data );
/* This function allows to access the element's integer attributes. */
extern int element_get_attribute( Element obj, int attribute );
/* This function allows to access the element's data. */
extern void * element_get_data( Element obj );
/* This function allows to access the element's draw function. */
extern Element_draw_function element_get_draw_function( Element obj );
/* This function allows to quickly move an element. */
void element_move( Element obj, int x_offset, int y_offset );


#endif
